﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace K02ADSample
{
    public partial class DevForm : Form
    {
        public short DevId;                     //Device Id
        private byte[] IOStatus = new byte[2];  //DIO Status
        private int TotalLeng;                 //Sampled Data Length
        private short[] dat1 = new short[2097152];//Data Buffer Ch1(dynamic memory allocation)
        private short[] dat2 = new short[2097152];//Data Buffer Ch2(dynamic memory allocation)
        private BinaryWriter SaveFile;          //Save File Stream
        public DevForm()
        {
            InitializeComponent();
        }

        public void SetId(short Id)
        {
            byte[] data = new byte[2];
            short retErr;

            DevId = Id;

            this.Text = "ID = " + DevId.ToString() + "  Panel";
            retErr = TUSBK02AD.TUSBK02AD_DIO_Status(DevId, IOStatus);
            if (retErr != 0)
            {
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                this.Hide();
                return;
            }
            else
            {
                IOSEL1.SelectedIndex = IOStatus[0];
                IOSEL2.SelectedIndex = IOStatus[1];
            }

            ExtClock.Text = "1000";
            IntClock.Text = "100000";
            TriggerType.SelectedIndex = 0;
            TriggerCh.SelectedIndex = 0;
            SEL1.Checked = true;
            SEL2.Checked = true;
            SelIntClk.Checked = true;
            Timer1.Interval = 100;
            Timer1.Enabled = true;
            RecStop.Enabled = false;
        }

        private void DevForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (Timer2.Enabled)
            {
                Timer2.Enabled = false;
                TUSBK02AD.TUSBK02AD_Ad_Stop(DevId);
            }
            Timer1.Enabled = false;
        }

        private void ExtClock_TextChanged(object sender, EventArgs e)
        {
            int tmpInt;

            if (int.TryParse(ExtClock.Text, out tmpInt))
            {
                if (tmpInt >= 1000 && tmpInt <= 500000000)
                    ExtClockDisp.Text = TUSBK02AD.TUSBK02AD_ExtTimer(tmpInt).ToString("N0");
                else
                    ExtClockDisp.Text = "";
            }
            else
                ExtClockDisp.Text = "";
        }

        private void IntClock_TextChanged(object sender, EventArgs e)
        {
            int tmpInt;

            if (int.TryParse(IntClock.Text, out tmpInt))
            {
                if( tmpInt >= 50000 && tmpInt <= 1000000000 )
                    IntClockDisp.Text = TUSBK02AD.TUSBK02AD_IntClk(tmpInt).ToString("N0");
                else
                    IntClockDisp.Text = "";
            }
            else
                IntClockDisp.Text = "";
        }

        private void ClockOut_Click(object sender, EventArgs e)
        {
            int Clock;
            short retErr;

            if (int.TryParse(ExtClock.Text, out Clock))
            {
                if (Clock < 1000 || Clock > 500000000)
                {
                    MessageBox.Show("Cycle is outside the range.");
                }
                else
                {
                    retErr = TUSBK02AD.TUSBK02AD_ExtTimer_Set(DevId, Clock);
                    if (retErr != 0)
                        MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                    else
                        IOSEL2.SelectedIndex = 3;
                }
            }
            else
                MessageBox.Show("An input value is invalid.");
        }

        private void SetDir(byte b)
        {
            byte dir;
            short retErr;

            dir = (IOStatus[b] != 0) ? (byte) 1: (byte) 0;

            retErr = TUSBK02AD.TUSBK02AD_DIO_Dir(DevId,b,dir);
            if( retErr != 0 )
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
        }

        private void SetDio(byte b)
        {
            byte data;
            short retErr;

            data = (IOStatus[b] == 2 ) ? (byte) 1 : (byte) 0;

            retErr = TUSBK02AD.TUSBK02AD_DIO_Out(DevId,b,data);
            if(retErr != 0)
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
        }

        private void IOSEL1_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (IOSEL1.SelectedIndex)
            {
                case 0://INPUT
                    IOStatus[0] = (byte) IOSEL1.SelectedIndex;
                    SetDir(0);
                    break;
                case 1://OUTPUT Low
                    IOStatus[0] = (byte) IOSEL1.SelectedIndex;
                    SetDio(0);
                    SetDir(0);
                    break;
                case 2://OUTPUT High
                    IOStatus[0] = (byte) IOSEL1.SelectedIndex;
                    SetDio(0);
                    SetDir(0);
                    break;
            }
        }

        private void IOSEL2_SelectedIndexChanged(object sender, EventArgs e)
        {
            short retErr;

            switch (IOSEL2.SelectedIndex)
            {
                case 0://INPUT
                    IOStatus[1] = (byte)IOSEL2.SelectedIndex;
                    SetDir(1);
                    break;
                case 1://OUTPUT Low
                    IOStatus[1] = (byte)IOSEL2.SelectedIndex;
                    SetDio(1);
                    SetDir(1);
                    break;
                case 2://OUTPUT High
                    IOStatus[1] = (byte) IOSEL2.SelectedIndex;
                    SetDio(1);
                    SetDir(1);
                    break;
                case 3://Clock
                    retErr = TUSBK02AD.TUSBK02AD_DIO_Status(DevId, IOStatus);
                    if (retErr != 0)
                    {
                        MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                        this.Hide();
                        return;
                    }
                    else if (IOStatus[1] != 3)
                        IOSEL2.SelectedIndex = IOStatus[1];
                    break;
            }
        }

        private void RecStart_Click(object sender, EventArgs e)
        {
            int Clock;
            byte chsel;
            short retErr;
            short Level;//Trigger Level
            short Hys;//Hysterisys Level
            int PreLen;//Pre Trigger Len
            double tmpD;

            if (SelIntClk.Checked)
            {
                if (int.TryParse(IntClock.Text, out Clock))
                {
                    if (Clock < 50000 || Clock > 1000000000)
                    {
                        MessageBox.Show("Cycle is outside the range.");
                        return;
                    }
                }
                else
                {
                    MessageBox.Show("An clock input value is invalid.");
                    return;
                }
            }
            else
                Clock = 0;

            // Trigger Level Check
            if (TriggerType.SelectedIndex >= 4)
            {
                if (double.TryParse(TriggerLevel.Text,out tmpD))
                {
                    if (tmpD < -2.5 || tmpD > 2.5)
                    {
                        MessageBox.Show("Trigger Level is outside the range.");
                        return;
                    }
                    Level = (short) Math.Round((tmpD + 2.5) / 5.0 * 4096.0);
                    if(Level < 0 )
                        Level = 0;
                    if(Level > 4095 )
                        Level = 4095;
                }
                else
                {
                    MessageBox.Show("Trigger Level is invalid.");
                    return;
                }
            }
            else
            {
                Level = 0;
            }

            // Hysterisys Level Check
            if (TriggerType.SelectedIndex >= 4)
            {
                if (double.TryParse(HysLevel.Text, out tmpD))
                {
                    if (tmpD < 0.0 || tmpD > 50.0)
                    {
                        MessageBox.Show("Hysterisys Level is outside the range.");
                        return;
                    }
                    Hys = (short)Math.Round(tmpD / 5000.0 * 4096.0);
                    if (Hys < 0)
                        Hys = 0;
                    if (Hys > 41)
                        Hys = 41;
                }
                else
                {
                    MessageBox.Show("Hysterisys Level is invalid.");
                    return;
                }
            }
            else
                Hys = 0;

            // Pre Trigger Len. Check
            if (int.TryParse(PreTriggerLen.Text, out PreLen))
            {
                if (PreLen < 0 || PreLen > 1048576)
                {
                    MessageBox.Show("Pre Trigger Len. is outside the range.");
                    return;
                }
            }
            else
            {
                MessageBox.Show("Pre Trigger Len. is invalid");
                return;
            }

            //Acquisition of a save file name
            SaveFileDialog1.Filter = "Data File (*.dat)|*.dat";
            if (SaveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                if (File.Exists(SaveFileDialog1.FileName))
                {
                    File.Delete(SaveFileDialog1.FileName);
                }
                SaveFile = new BinaryWriter(File.OpenWrite(SaveFileDialog1.FileName));//Open File

                chsel = (SEL1.Checked) ? ((SEL2.Checked) ? (byte)2 : (byte)0) : (byte)1;

                retErr = TUSBK02AD.TUSBK02AD_Trigger_Set(DevId,(byte)TriggerType.SelectedIndex, Level, Hys, PreLen);
                if (retErr != 0)
                {
                    SaveFile.Close();
                    MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                    return;
                }

                retErr = TUSBK02AD.TUSBK02AD_Ad_Start(DevId, chsel, (byte)TriggerCh.SelectedIndex, Clock);

                if (retErr == 0)
                {
                    TotalLeng = 0;
                    Timer2.Interval = 100;
                    Timer2.Enabled = true;
                    EnableDisableCont(1);
                }
                else
                {
                    SaveFile.Close();
                    MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                }
            }
        }

        //Enable Disable change of controls
        private void EnableDisableCont(byte StartStop)
        {
            if( StartStop != 0 )//Start ( on data recording )
            {
                RecStop.Enabled = true;
                RecStart.Enabled = false;
                SingleSample.Enabled = false;
                TriggerType.Enabled = false;
                SEL1.Enabled = false;
                SEL2.Enabled = false;
                SelExtClock.Enabled = false;
                SelIntClk.Enabled = false;
                PreTriggerLen.Enabled = false;

                IntClock.Enabled = false;

                TriggerCh.Enabled = false;
                TriggerLevel.Enabled = false;
                HysLevel.Enabled = false; 
            }
            else
            {
                RecStop.Enabled = false;
                RecStart.Enabled = true;
                SingleSample.Enabled = true;
                TriggerType.Enabled = true;
                SEL1.Enabled = true;
                SEL2.Enabled = true;
                SelExtClock.Enabled = true;
                SelIntClk.Enabled = true;
                PreTriggerLen.Enabled = true;

                if( SelIntClk.Checked )
                    IntClock.Enabled = true;

                TriggerType_Click();            
            }
        }

        //single sample command
        private void SingleSample_Click(object sender, EventArgs e)
        {
            short dataI;
            double dataD;
            short retErr;

            retErr = TUSBK02AD.TUSBK02AD_Ad_Single(DevId, 0, out dataI);
            if (retErr != 0)
            {
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                return;
            } 
            dataD = dataI;
            dataD = (dataD - 2048.0) / 2048.0 * 2.5;
            AD_DATA_1.Text = dataD.ToString("0.000");

            retErr = TUSBK02AD.TUSBK02AD_Ad_Single(DevId, 1, out dataI);
            if (retErr != 0)
            {
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                return;
            } 
            dataD = dataI;
            dataD = (dataD - 2048.0) / 2048.0 * 2.5;
            AD_DATA_2.Text = dataD.ToString("0.000");
        }

        //Trigger Type select command
        private void TriggerType_Click()
        {
            switch (TriggerType.SelectedIndex)
            {
                case 0:
                case 1:
                case 2:
                case 3:
                    TriggerCh.Enabled = false;
                    TriggerLevel.Enabled = false;
                    HysLevel.Enabled = false;
                    break;
                case 4:
                case 5:
                    TriggerCh.Enabled = true;
                    TriggerLevel.Enabled = true;
                    HysLevel.Enabled = true;
                    break;
                case 6:
                case 7:
                case 8:
                case 9:
                    TriggerCh.Enabled = true;
                    TriggerLevel.Enabled = true;
                    HysLevel.Enabled = false;
                    break;
            }
        }

        private void TriggerType_SelectedIndexChanged(object sender, EventArgs e)
        {
            TriggerType_Click();
        }

        // stop command of data recording
        private void RecordStop()
        {
            short retErr;
            int DataLen;
            int cnt;

            Timer2.Enabled = false;

            retErr = TUSBK02AD.TUSBK02AD_Ad_Stop(DevId);

            if (retErr != 0)
            {
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                SaveFile.Close();
                return;
            }

            DataLen = 2097152;
            retErr = TUSBK02AD.TUSBK02AD_Get_Data(DevId, dat1, dat2, ref DataLen);
            if (retErr != 0)
            {
                Timer2.Enabled = false;// Timer1 is suspended.
                SaveFile.Close();
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                EnableDisableCont(0);
                return;
            }

            for (cnt = 0; cnt < DataLen; cnt++)
                SaveFile.Write(dat1[cnt].ToString() + "," + dat2[cnt].ToString() + "\r\n");

            SaveFile.Close();

            TotalLeng = TotalLeng + DataLen;
            SamplingStatusDisp.Text = "TotalLeng = " + TotalLeng.ToString();

            EnableDisableCont(0);
        }

        private void RecStop_Click(object sender, EventArgs e)
        {
            RecordStop();
        }

        private void SelExtClock_CheckedChanged(object sender, EventArgs e)
        {
            IntClock.Enabled = false;
        }

        private void SelIntClk_CheckedChanged(object sender, EventArgs e)
        {
            IntClock.Enabled = true;
        }

        private void SoftTrigger_Click(object sender, EventArgs e)
        {
            short retErr;
            retErr = TUSBK02AD.TUSBK02AD_SoftTrigger(DevId);
            if (retErr != 0)
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            DIO_Check();
        }

        private void DIO_Check()
        {
            byte data;
            short retErr;

            retErr = TUSBK02AD.TUSBK02AD_DIO_In(DevId, out data);
            if (retErr != 0)
            {
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                this.Hide();
                return;
            }

            if ((data & 0x01) != 0)
                D0State.Text = "High";
            else
                D0State.Text = "Low";

            if ((data & 0x02) != 0)
                D1State.Text = "High";
            else
                D1State.Text = "Low";
        }

        private void Timer2_Tick(object sender, EventArgs e)
        {
            byte AdStatus;
            int DataLen;
            short retErr;
            int cnt;

            retErr = TUSBK02AD.TUSBK02AD_Get_Status(DevId, out AdStatus, out DataLen);

            if (retErr != 0)
            {
                Timer2.Enabled = false; //Timer2 is suspended.
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                RecordStop();
                return;
            }

            retErr = TUSBK02AD.TUSBK02AD_Get_Data(DevId, dat1, dat2, ref DataLen);

            if (retErr != 0)
            {
                Timer2.Enabled = false; //Timer2 is suspended.
                MessageBox.Show(TUSBK02AD.GetErrMessage(retErr));
                RecordStop();
                return;
            }

            for (cnt = 0; cnt < DataLen; cnt++)
                SaveFile.Write(dat1[cnt].ToString() + "," + dat2[cnt].ToString() + "\r\n");

            TotalLeng = TotalLeng + DataLen;
            SamplingStatusDisp.Text = "Status = " + AdStatus.ToString() + "   TotalLeng = " + TotalLeng.ToString();
        }
    }
}
