﻿Imports System.IO

Public Class DevForm
    Public DevId As Short           'Device Id
    Private IOStatus(1) As Byte     'DIO Status
    Private TotalLeng As Long       'Sampled Data Length
    Private dat1(2097151) As Short         'Data Buffer Ch1(dynamic memory allocation)
    Private dat2(2097151) As Short         'Data Buffer Ch2(dynamic memory allocation)
    Private SaveFile As BinaryWriter 'Save File Stream

    'This Form module initialize
    Public Sub SetId(ByVal Id As Short)
        Dim data(2) As Byte
        Dim retErr As Short

        DevId = Id

        Me.Text = "ID = " + DevId.ToString() + "  Panel"
        retErr = TUSBK02AD_DIO_Status(DevId, IOStatus)
        If retErr <> 0 Then
            TUSBK02AD_ErrorDisp(retErr)
            Me.Hide()
            Return
        Else
            IOSEL1.SelectedIndex = IOStatus(0)
            IOSEL2.SelectedIndex = IOStatus(1)
        End If

        ExtClock.Text = "1000"
        IntClock.Text = "100000"
        TriggerType.SelectedIndex = 0
        TriggerCh.SelectedIndex = 0
        SEL1.Checked() = True
        SEL2.Checked() = True
        SelIntClk.Checked() = True
        Timer1.Interval = 100
        Timer1.Enabled = True
        RecStop.Enabled = False
    End Sub

    Private Sub DevForm_FormClosed(sender As System.Object, e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        If Timer2.Enabled Then
            Timer2.Enabled = False
            TUSBK02AD_Ad_Stop(DevId)
        End If
        Timer1.Enabled = False
    End Sub

    Private Sub ExtClock_TextChanged(sender As System.Object, e As System.EventArgs) Handles ExtClock.TextChanged
        Dim tmpInt As Integer

        If (Integer.TryParse(ExtClock.Text, tmpInt)) Then
            If (tmpInt >= 1000 And tmpInt <= 500000000) Then
                ExtClockDisp.Text = TUSBK02AD_ExtTimer(tmpInt).ToString("N0")
            Else
                ExtClockDisp.Text = ""
            End If
        Else
            ExtClockDisp.Text = ""
        End If
    End Sub

    Private Sub IntClock_TextChanged(sender As System.Object, e As System.EventArgs) Handles IntClock.TextChanged
        Dim tmpInt As Integer

        If (Integer.TryParse(IntClock.Text, tmpInt)) Then
            If (tmpInt >= 50000 And tmpInt <= 1000000000) Then
                IntClockDisp.Text = TUSBK02AD_IntClk(tmpInt).ToString("N0")
            Else
                IntClockDisp.Text = ""
            End If
        Else
            IntClockDisp.Text = ""
        End If
    End Sub

    ' Ext Clock Out Command
    Private Sub ClockOut_Click(sender As System.Object, e As System.EventArgs) Handles ClockOut.Click
        Dim Clock As Integer
        Dim retErr As Short

        If Integer.TryParse(ExtClock.Text, Clock) Then
            If (Clock < 1000 Or Clock > 500000000) Then
                MessageBox.Show("Cycle is outside the range.")
            Else
                retErr = TUSBK02AD_ExtTimer_Set(DevId, Clock)
                If retErr <> 0 Then
                    TUSBK02AD_ErrorDisp(retErr)
                Else
                    IOSEL2.SelectedIndex = 3
                End If
            End If
        Else
            MessageBox.Show("An input value is invalid.")
        End If
    End Sub

    ' Digital Port Direction Change
    Private Sub SetDir(ByVal b As Byte)
        Dim dir As Byte
        Dim retErr As Short

        dir = IIf(IOStatus(b) <> 0, 1, 0)

        retErr = TUSBK02AD_DIO_Dir(DevId, b, dir)
        If retErr Then
            TUSBK02AD_ErrorDisp(retErr)
        End If
    End Sub

    'Digital Port Output Level
    Private Sub SetDio(ByVal b As Byte)
        Dim data As Byte
        Dim retErr As Short

        data = IIf(IOStatus(b) = 2, 1, 0)

        retErr = TUSBK02AD_DIO_Out(DevId, b, data)
        If retErr Then
            TUSBK02AD_ErrorDisp(retErr)
        End If
    End Sub

    'setup of Digital IO 1 state
    Private Sub IOSEL1_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles IOSEL1.SelectedIndexChanged
        Select Case IOSEL1.SelectedIndex
            Case 0 'INPUT
                IOStatus(0) = IOSEL1.SelectedIndex
                SetDir(0)
            Case 1 'OUTPUT Low
                IOStatus(0) = IOSEL1.SelectedIndex
                SetDio(0)
                SetDir(0)
            Case 2 'OUTPUT High
                IOStatus(0) = IOSEL1.SelectedIndex
                SetDio(0)
                SetDir(0)
        End Select
    End Sub

    Private Sub IOSEL2_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles IOSEL2.SelectedIndexChanged
        Dim retErr As Short

        Select Case IOSEL2.SelectedIndex
            Case 0 'INPUT
                IOStatus(1) = IOSEL2.SelectedIndex
                SetDir(1)
            Case 1 'OUTPUT Low
                IOStatus(1) = IOSEL2.SelectedIndex
                SetDio(1)
                SetDir(1)
            Case 2 'OUTPUT High
                IOStatus(1) = IOSEL2.SelectedIndex
                SetDio(1)
                SetDir(1)
            Case 3 'CLOCK
                retErr = TUSBK02AD_DIO_Status(DevId, IOStatus)
                If retErr <> 0 Then
                    TUSBK02AD_ErrorDisp(retErr)
                    Me.Hide()
                    Return
                ElseIf IOStatus(1) <> 3 Then
                    IOSEL2.SelectedIndex = IOStatus(1)
                End If
        End Select
    End Sub

    ' start command of data recording
    Private Sub RecStart_Click(sender As System.Object, e As System.EventArgs) Handles RecStart.Click
        Dim Clock As Integer
        Dim chsel As Byte
        Dim retErr As Short
        Dim Level As Short 'Trigger Level
        Dim Hys As Short 'Hysterisys Level
        Dim PreLen As Integer 'Pre Trigger Len
        Dim tmpD As Double

        If SelIntClk.Checked Then
            If Integer.TryParse(IntClock.Text, Clock) Then
                If (Clock < 50000 Or Clock > 1000000000) Then
                    MessageBox.Show("Cycle is outside the range.")
                    Return
                End If
            Else
                MessageBox.Show("An clock input value is invalid.")
                Return
            End If
        Else
            Clock = 0
        End If

        ' Trigger Level Check
        If TriggerType.SelectedIndex >= 4 Then
            If Double.TryParse(TriggerLevel.Text, tmpD) Then
                If (tmpD < -2.5 Or tmpD > 2.5) Then
                    MessageBox.Show("Trigger Level is outside the range.")
                    Return
                End If
                Level = Math.Round((tmpD + 2.5) / 5.0# * 4096.0#)
                If Level < 0 Then
                    Level = 0
                End If
                If Level > 4095 Then
                    Level = 4095
                End If
            Else
                MessageBox.Show("Trigger Level is invalid.")
                Return
            End If
        Else
            Level = 0
        End If

        ' Hysterisys Level Check
        If TriggerType.SelectedIndex >= 4 Then
            If Double.TryParse(HysLevel.Text, tmpD) Then
                If (tmpD < 0.0# Or tmpD > 50.0#) Then
                    MessageBox.Show("Hysterisys Level is outside the range.")
                    Return
                End If
                Hys = tmpD / 5000.0# * 4096.0#
                If Hys < 0 Then
                    Hys = 0
                End If
                If Hys > 41 Then
                    Hys = 41
                End If
            Else
                MessageBox.Show("Hysterisys Level is invalid.")
                Return
            End If
        Else
            Hys = 0
        End If

        ' Pre Trigger Len. Check
        If Integer.TryParse(PreTriggerLen.Text, PreLen) Then
            If PreLen < 0 Or PreLen > 1048576 Then
                MessageBox.Show("Pre Trigger Len. is outside the range.")
                Return
            End If
        Else
            MessageBox.Show("Pre Trigger Len. is invalid")
            Return
        End If

        'Acquisition of a save file name
        SaveFileDialog1.Filter = "Data File (*.dat)|*.dat"
        If SaveFileDialog1.ShowDialog() = DialogResult.OK Then

            If (File.Exists(SaveFileDialog1.FileName)) Then
                File.Delete(SaveFileDialog1.FileName)
            End If
            SaveFile = New BinaryWriter(File.OpenWrite(SaveFileDialog1.FileName)) 'Open File

            chsel = IIf(SEL1.Checked, IIf(SEL2.Checked, 2, 0), 1)

            retErr = TUSBK02AD_Trigger_Set(DevId, TriggerType.SelectedIndex, Level, Hys, PreLen)
            If retErr <> 0 Then
                SaveFile.Close()
                TUSBK02AD_ErrorDisp(retErr)
                Return
            End If

            retErr = TUSBK02AD_Ad_Start(DevId, chsel, TriggerCh.SelectedIndex, Clock)
            If retErr = 0 Then
                TotalLeng = 0
                Timer2.Interval = 100
                Timer2.Enabled = True
                EnableDisableCont(1)
            Else
                SaveFile.Close()
                TUSBK02AD_ErrorDisp(retErr)
            End If
        End If
    End Sub

    ' Enable Disable change of controls
    Private Sub EnableDisableCont(ByVal StartStop As Byte)
        If StartStop Then 'Start ( on data recording )
            RecStop.Enabled = True
            RecStart.Enabled = False
            SingleSample.Enabled = False
            TriggerType.Enabled = False
            SEL1.Enabled = False
            SEL2.Enabled = False
            SelExtClock.Enabled = False
            SelIntClk.Enabled = False
            PreTriggerLen.Enabled = False

            IntClock.Enabled = False

            TriggerCh.Enabled = False
            TriggerLevel.Enabled = False
            HysLevel.Enabled = False
        Else 'Stop
            RecStop.Enabled = False
            RecStart.Enabled = True
            SingleSample.Enabled = True
            TriggerType.Enabled = True
            SEL1.Enabled = True
            SEL2.Enabled = True
            SelExtClock.Enabled = True
            SelIntClk.Enabled = True
            PreTriggerLen.Enabled = True

            If SelIntClk.Checked Then
                IntClock.Enabled = True
            End If

            TriggerType_Click()
        End If
    End Sub

    'single sample command
    Private Sub SingleSample_Click(sender As System.Object, e As System.EventArgs) Handles SingleSample.Click
        Dim dataI As Short
        Dim dataD As Double
        Dim retErr As Short

        retErr = TUSBK02AD_Ad_Single(DevId, 0, dataI)
        If retErr <> 0 Then
            TUSBK02AD_ErrorDisp(retErr)
            Return
        End If
        dataD = dataI
        dataD = (dataD - 2048.0#) / 2048.0# * 2.5
        AD_DATA_1.Text = dataD.ToString("0.000")

        retErr = TUSBK02AD_Ad_Single(DevId, 1, dataI)
        If retErr <> 0 Then
            TUSBK02AD_ErrorDisp(retErr)
            Return
        End If
        dataD = dataI
        dataD = (dataD - 2048.0#) / 2048.0# * 2.5
        AD_DATA_2.Text = Format(dataD, "0.000")
    End Sub

    'Trigger Type select command
    Private Sub TriggerType_Click()
        Select Case TriggerType.SelectedIndex
            Case 0 To 3
                TriggerCh.Enabled = False
                TriggerLevel.Enabled = False
                HysLevel.Enabled = False
            Case 4, 5
                TriggerCh.Enabled = True
                TriggerLevel.Enabled = True
                HysLevel.Enabled = True
            Case 6 To 9
                TriggerCh.Enabled = True
                TriggerLevel.Enabled = True
                HysLevel.Enabled = False
        End Select
    End Sub

    Private Sub TriggerType_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles TriggerType.SelectedIndexChanged
        TriggerType_Click()
    End Sub

    ' stop command of data recording
    Private Sub RecordStop()

        Dim retErr As Short
        Dim DataLen As Integer
        Dim cnt As Integer

        Timer2.Enabled = False

        retErr = TUSBK02AD_Ad_Stop(DevId)
        If retErr <> 0 Then
            TUSBK02AD_ErrorDisp(retErr)
            SaveFile.Close()
            Return
        End If

        DataLen = 2097152
        retErr = TUSBK02AD_Get_Data(DevId, dat1, dat2, DataLen)
        If retErr <> 0 Then
            Timer2.Enabled = False ' Timer1 is suspended.
            SaveFile.Close()
            TUSBK02AD_ErrorDisp(retErr)
            EnableDisableCont(0)
            Return
        End If

        For cnt = 0 To (DataLen - 1)
            SaveFile.Write(dat1(cnt).ToString() & "," & dat2(cnt).ToString() & vbCrLf)
        Next cnt

        SaveFile.Close()

        TotalLeng = TotalLeng + DataLen
        SamplingStatusDisp.Text = "TotalLeng = " & TotalLeng.ToString()

        EnableDisableCont(0)
    End Sub

    Private Sub RecStop_Click(sender As System.Object, e As System.EventArgs) Handles RecStop.Click
        RecordStop()
    End Sub

    Private Sub SelExtClock_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles SelExtClock.CheckedChanged
        IntClock.Enabled = False
    End Sub

    Private Sub SelIntClk_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles SelIntClk.CheckedChanged
        IntClock.Enabled = True
    End Sub

    Private Sub SoftTrigger_Click(sender As System.Object, e As System.EventArgs) Handles SoftTrigger.Click
        Dim retErr As Short
        retErr = TUSBK02AD_SoftTrigger(DevId)
        If retErr <> 0 Then
            TUSBK02AD_ErrorDisp(retErr)
        End If
    End Sub

    Private Sub Timer1_Tick(sender As System.Object, e As System.EventArgs) Handles Timer1.Tick
        DIO_Check()
    End Sub

    Private Sub DIO_Check()
        Dim data As Byte
        Dim retErr As Short

        retErr = TUSBK02AD_DIO_In(DevId, data)
        If retErr <> 0 Then
            TUSBK02AD_ErrorDisp(retErr)
            Me.Hide()
            Return
        End If
        If data And &H1 Then
            D0State.Text = "High"
        Else
            D0State.Text = "Low"
        End If
        If data And &H2 Then
            D1State.Text = "High"
        Else
            D1State.Text = "Low"
        End If
    End Sub

    Private Sub Timer2_Tick(sender As System.Object, e As System.EventArgs) Handles Timer2.Tick
        Dim AdStatus As Byte
        Dim DataLen As Long
        Dim retErr As Short
        Dim cnt As Integer

        retErr = TUSBK02AD_Get_Status(DevId, AdStatus, DataLen)
        If retErr <> 0 Then
            Timer2.Enabled = False 'Timer2 is suspended.
            TUSBK02AD_ErrorDisp(retErr)
            RecordStop()
            Return
        End If

        retErr = TUSBK02AD_Get_Data(DevId, dat1, dat2, DataLen)
        If retErr <> 0 Then
            Timer2.Enabled = False 'Timer2 is suspended.
            TUSBK02AD_ErrorDisp(retErr)
            RecordStop()
            Return
        End If

        For cnt = 0 To (DataLen - 1)
            SaveFile.Write(dat1(cnt).ToString() & "," & dat2(cnt).ToString() & vbCrLf)
        Next cnt

        TotalLeng = TotalLeng + DataLen
        SamplingStatusDisp.Text = "Status = " & AdStatus.ToString() & "   TotalLeng = " & TotalLeng.ToString()
    End Sub
End Class